@@ -3,8 +3,8 @@
  *
  * Derived from Linux version by Richard Tobin.
  *
- * $Id: bsd.c,v 1.28 2003/12/30 01:12:38 jerdfelt Exp $
- * $Name:  $
+ * $Id: bsd.c,v 1.30 2004/02/18 06:34:52 jerdfelt Exp $
+ * $Name: V0_1_10 $
  *
  * This library is covered by the LGPL, read LICENSE for details.
  */
@@ -278,7 +278,7 @@ static int ensure_ep_open(usb_dev_handle *dev, int ep, int mode)
 int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
                    int timeout)
 {
-  int fd, ret, sent = 0;
+  int fd, ret;
 
   /* Ensure the endpoint address is correct */
   ep &= ~USB_ENDPOINT_IN;
@@ -300,27 +300,23 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
     USB_ERROR_STR(-errno, "error setting timeout: %s",
                   strerror(errno));
 
-  do {
-    ret = write(fd, bytes+sent, size-sent);
-    if (ret < 0)
+  ret = write(fd, bytes, size);
+  if (ret < 0)
 #if __FreeBSD__
-      USB_ERROR_STR(-errno, "error writing to bulk endpoint %s.%d: %s",
-                    dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
-#else
-      USB_ERROR_STR(-errno, "error writing to bulk endpoint %s.%02d: %s",
+    USB_ERROR_STR(-errno, "error writing to bulk endpoint %s.%d: %s",
                   dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
+#else
+    USB_ERROR_STR(-errno, "error writing to bulk endpoint %s.%02d: %s",
+                dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #endif
 
-    sent += ret;
-  } while (ret > 0 && sent < size);
-
-  return sent;
+  return size;
 }
 
 int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
                   int timeout)
 {
-  int fd, ret, retrieved = 0, one = 1;
+  int fd, ret, one = 1;
 
   /* Ensure the endpoint address is correct */
   ep |= USB_ENDPOINT_IN;
@@ -345,20 +341,17 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
   if (ret < 0)
     USB_ERROR_STR(-errno, "error setting short xfer: %s", strerror(errno));
 
-  do {
-    ret = read(fd, bytes+retrieved, size-retrieved);
-    if (ret < 0)
+  ret = read(fd, bytes, size);
+  if (ret < 0)
 #if __FreeBSD__
-      USB_ERROR_STR(-errno, "error reading from bulk endpoint %s.%d: %s",
-                    dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
-#else
-      USB_ERROR_STR(-errno, "error reading from bulk endpoint %s.%02d: %s",
+    USB_ERROR_STR(-errno, "error reading from bulk endpoint %s.%d: %s",
                   dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
+#else
+    USB_ERROR_STR(-errno, "error reading from bulk endpoint %s.%02d: %s",
+                dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #endif
-    retrieved += ret;
-  } while (ret > 0 && retrieved < size);
 
-  return retrieved;
+  return ret;
 }
 
 int usb_interrupt_write(usb_dev_handle *dev, int ep, char *bytes, int size,
@@ -608,6 +601,12 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
   return 0;
 }
 
+int usb_os_determine_children(struct usb_bus *bus)
+{
+  /* Nothing yet */
+  return 0;
+}
+
 void usb_os_init(void)
 {
   /* nothing */
