@@ -41,15 +41,18 @@
 #include "config.h"
 #endif
 
-#include "usbi.h"
-
+#include <stdlib.h>
+#include <stdio.h>
 #include <unistd.h>
 
 /* standard includes for darwin/os10 (IOKit) */
+#include <mach/mach_port.h>
 #include <IOKit/IOCFBundle.h>
 #include <IOKit/usb/IOUSBLib.h>
 #include <IOKit/IOCFPlugIn.h>
 
+#include "usbi.h"
+
 /* some defines */
 /* IOUSBInterfaceInferface */
 #if defined (IOUSBINTERFACE_FUNCS_197)
@@ -113,7 +116,9 @@
 
 #endif
 
-#define io_return_t     IOReturn
+typedef IOReturn io_return_t;
+typedef IOCFPlugInInterface *io_cf_plugin_ref_t;
+typedef SInt32 s_int32_t;
 
 /* Darwin/OS X impl does not use fd field, instead it uses this */
 struct darwin_dev_handle {
@@ -122,7 +127,6 @@ struct darwin_dev_handle {
   int open;
 };
 
-static CFRunLoopSourceRef runLoopSource;
 static CFMutableDictionaryRef matchingDict;
 static IONotificationPortRef gNotifyPort;
 static mach_port_t masterPort = MACH_PORT_NULL;
@@ -187,15 +191,13 @@ int usb_os_open(usb_dev_handle *dev)
   io_return_t result;
   io_iterator_t deviceIterator;
   io_service_t usbDevice;
+  io_cf_plugin_ref_t *plugInInterface = NULL;
 
   usb_device_t **darwin_device;
 
-  IOCFPlugInInterface **plugInInterface = NULL;
-
-  long score;
-  UInt16 address, vendor;
-  UInt32 location = *((UInt32 *)dev->device->dev);
-  UInt32 dlocation;
+  s_int32_t score;
+  u_int32_t location = *((u_int32_t *)dev->device->dev);
+  u_int32_t dlocation;
 
   if (!dev)
     USB_ERROR(-ENXIO);
@@ -248,7 +250,7 @@ int usb_os_open(usb_dev_handle *dev)
     if (!darwin_device)
       continue;
 
-    result = (*(darwin_device))->GetLocationID(darwin_device, (UInt32 *)&dlocation);
+    result = (*(darwin_device))->GetLocationID(darwin_device, (u_int32_t *)&dlocation);
     if (dlocation == location) {
       device->device = darwin_device;
       break;
@@ -259,6 +261,9 @@ int usb_os_open(usb_dev_handle *dev)
 
   IOObjectRelease(deviceIterator);
 
+  if (device->device == NULL)
+    USB_ERROR_STR (-ENOENT, "usb_os_open: %s\n", "Device not found!");
+
 #if !defined (LIBUSB_NO_SEIZE_DEVICE)
   result = (*(device->device))->USBDeviceOpenSeize (device->device);
 #else
@@ -322,16 +327,16 @@ int usb_os_close(usb_dev_handle *dev)
   return 0;
 }
 
-// static int claim_interface ( struct darwin_dev_handle *device, int interface )
 static int claim_interface ( usb_dev_handle *dev, int interface )
 {
   io_iterator_t interface_iterator;
   io_service_t  usbInterface;
+  io_return_t result;
+  io_cf_plugin_ref_t *plugInInterface = NULL;
+
+  IOUSBFindInterfaceRequest request;
 
   struct darwin_dev_handle *device;
-  io_return_t result;
-  IOUSBFindInterfaceRequest request;
-  IOCFPlugInInterface **plugInInterface = NULL;
   long score;
   int current_interface;
 
@@ -351,7 +356,7 @@ static int claim_interface ( usb_dev_handle *dev, int interface )
     usbInterface = IOIteratorNext(interface_iterator);
     if ( usb_debug > 3 )
       fprintf ( stderr, "Interface %d of device is 0x%x\n",
-		current_interface, (void *)usbInterface );
+		current_interface, usbInterface );
   }
   current_interface--;
 
@@ -359,7 +364,7 @@ static int claim_interface ( usb_dev_handle *dev, int interface )
   IOObjectRelease(interface_iterator);
 
   if (!usbInterface) {
-    UInt8 nConfig;			     /* Index of configuration to use */
+    u_int8_t nConfig;			     /* Index of configuration to use */
     IOUSBConfigurationDescriptorPtr configDesc; /* to describe which configuration to select */
     /* Only a composite class device with no vendor-specific driver will
        be configured. Otherwise, we need to do it ourselves, or there
@@ -425,7 +430,7 @@ static int claim_interface ( usb_dev_handle *dev, int interface )
 
       if ( usb_debug > 3 )
 	fprintf ( stderr, "claim_interface: Interface %d of device is 0x%x\n",
-		  current_interface, (void *)usbInterface );
+		  current_interface, usbInterface );
     }
     current_interface--;
 
@@ -440,17 +445,17 @@ static int claim_interface ( usb_dev_handle *dev, int interface )
 					     kIOUSBInterfaceUserClientTypeID,
 					     kIOCFPlugInInterfaceID,
 					     &plugInInterface, &score);
-  //No longer need the usbInterface object after getting the plug-in
+  /* No longer need the usbInterface object after getting the plug-in */
   result = IOObjectRelease(usbInterface);
   if (result || !plugInInterface)
     USB_ERROR(-ENOENT);
 
-  //Now create the device interface for the interface
+  /* Now create the device interface for the interface */
   result = (*plugInInterface)->QueryInterface(plugInInterface,
 					      CFUUIDGetUUIDBytes(InterfaceInterfaceID),
 					      (LPVOID) &device->interface);
 
-  //No longer need the intermediate plug-in  
+  /* No longer need the intermediate plug-in */
   (*plugInInterface)->Stop(plugInInterface);
   IODestroyPlugInInterface (plugInInterface);
 
@@ -463,7 +468,7 @@ static int claim_interface ( usb_dev_handle *dev, int interface )
 
   if ( usb_debug > 3 )
     fprintf ( stderr, "claim_interface: Interface %d of device from QueryInterface is 0x%x\n",
-	      current_interface, (void *)(device->interface) );
+	      current_interface, device->interface);
 
   /* claim the interface */
   result = (*(device->interface))->USBInterfaceOpen(device->interface);
@@ -600,10 +605,13 @@ int usb_set_altinterface(usb_dev_handle *dev, int alternate)
 /* simple function that figures out what pipeRef is associated with an endpoint */
 static int ep_to_pipeRef (struct darwin_dev_handle *device, int ep)
 {
+  io_return_t ret;
+
   u_int8_t numep, direction, number;
   u_int8_t dont_care1, dont_care3;
   u_int16_t dont_care2;
-  int i, ret;
+
+  int i;
 
   if (usb_debug > 1)
     fprintf(stderr, "Converting ep address to pipeRef.\n");
@@ -646,10 +654,16 @@ static int ep_to_pipeRef (struct darwin_dev_handle *device, int ep)
   return -1;
 }
 
-static void write_completed(void *dummy, io_return_t result, void *arg0)
+static void rw_completed(void *ret_io_size, io_return_t result, void *io_size)
 {
   if (usb_debug > 2)
-    fprintf(stderr, "write completed\n");
+    fprintf(stderr, "io async operation completed. result = %i, size = %i\n", result,
+	    (u_int32_t)io_size);
+
+  if (result == kIOReturnSuccess)
+    *(u_int32_t *)ret_io_size = (u_int32_t)io_size;
+  else
+    *(u_int32_t *)ret_io_size = result;
 
   CFRunLoopStop(CFRunLoopGetCurrent());
 }
@@ -658,18 +672,21 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
                    int timeout)
 {
   struct darwin_dev_handle *device;
-  CFRunLoopSourceRef cfSource;
+
   io_return_t result = -1;
+
+  CFRunLoopSourceRef cfSource;
   int pipeRef;
 
-  UInt8   direction, number, transferType, interval;
-  UInt16			       maxPacketSize;
+  u_int8_t  direction, number, transferType, interval;
+  u_int16_t maxPacketSize;
+  u_int32_t written;
 
   if (!dev)
-    USB_ERROR(-ENXIO);
+    USB_ERROR_STR ( -ENXIO, "usb_bulk_read: called with null device" );
 
   if ((device = dev->impl_info) == NULL)
-    USB_ERROR(-ENOENT);
+    USB_ERROR_STR ( -ENOENT, "usb_bulk_read: device not initialized" );
 
   /* interface is not open */
   if (!device->interface)
@@ -689,38 +706,42 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
 
   /* there seems to be no way to determine how many bytes are actually written */
 #if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
-  if ( transferType == kUSBInterrupt )
+  if ( transferType != kUSBInterrupt )
     result = (*(device->interface))->WritePipeAsyncTO(device->interface, pipeRef,
 						      bytes, size, 0, timeout,
-						      write_completed, NULL);
+						      (IOAsyncCallback1)rw_completed,
+						      (void *)&written);
   else
 #endif
-    result = (*(device->interface))->WritePipeAsync(device->interface, pipeRef,
-						    bytes, size, write_completed, NULL);
+    result = (*(device->interface))->WritePipeAsync(device->interface, pipeRef, bytes,
+						    size, (IOAsyncCallback1)rw_completed,
+						    (void *)&written);
   
-  if (result != kIOReturnSuccess)
-    USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_write(WritePipeAsyncTO): %s", darwin_error_str(result));
-  else
+  if (result == kIOReturnSuccess)
     /* wait for write to complete */
     CFRunLoopRun();
 
   CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
 
+  if (result != kIOReturnSuccess)
+    USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_write(WritePipeAsyncTO): %s", darwin_error_str(result));
 
-  return size;
+  return written;
 }
 
 int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
                   int timeout)
 {
   struct darwin_dev_handle *device;
-  int pipeRef;
-  u_int32_t ret_size = size;
-  u_int32_t retrieved = 0;
   io_return_t result = -1;
 
-  UInt8				direction, number, transferType, interval;
-  UInt16				maxPacketSize;
+  int pipeRef;
+
+  u_int32_t retrieved = 0;
+  u_int16_t maxPacketSize;
+  u_int8_t  direction, number, transferType, interval;
+
+  CFRunLoopSourceRef cfSource;
 
   if (usb_debug)
     fprintf (stderr, "usb_bulk_read: ep addr = 0x%02x\n", ep);
@@ -741,40 +762,45 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
     USB_ERROR_STR ( -EINVAL, "usb_bulk_read: invalid pipe reference" );
 
   if (usb_debug > 3)
-    fprintf(stderr, "usb_bulk_read: endpoint=0x%02x size=%i timeout=%fsec\n" , ep, size, (double)timeout/1000 );
+    fprintf(stderr, "usb_bulk_read: endpoint=0x%02x size=%i timeout=%fsec\n" , ep, size, (double)timeout/1000.0 );
 
   (*(device->interface))->GetPipeProperties ( device->interface, pipeRef, &direction, &number,
 					     &transferType, &maxPacketSize, &interval );
 
+  (*(device->interface))->CreateInterfaceAsyncEventSource(device->interface, &cfSource);
+  CFRunLoopAddSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
+
 #if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
   if ( transferType == kUSBInterrupt ) {
 #endif
     /* This is an interrupt pipe. We can't specify a timeout. */
     if (usb_debug > 3)
       fprintf(stderr, "usb_bulk_read: interrupt pipe\n" );
-    result = (*(device->interface))->ReadPipe(device->interface, pipeRef,
-						  bytes, (UInt32 *)&ret_size);
-    if (result != kIOReturnSuccess)
-      USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_read(ReadPipe): error reading from bulk endpoint %02x: %s",
-		    ep, darwin_error_str(result));
-
-    retrieved = ret_size;
+    result = (*(device->interface))->ReadPipeAsync(device->interface, pipeRef, bytes,
+					      size, (IOAsyncCallback1)rw_completed,
+					      (void *)&retrieved);
 #if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
   } else {
-    result = (*(device->interface))->ReadPipeTO(device->interface, pipeRef,
-						bytes + retrieved, (UInt32 *)&ret_size, 0,
-						timeout);
+    result = (*(device->interface))->ReadPipeAsyncTO(device->interface, pipeRef,
+						     bytes + retrieved, size, 0,
+						     timeout, (IOAsyncCallback1)rw_completed,
+						     (void *)&retrieved);
     
     if (result != kIOReturnSuccess)
       USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_read(ReadPipeTO): error reading from bulk endpoint %02x: %s",
 		    ep, darwin_error_str(result));
-    retrieved = ret_size;
   }
 #endif
 
-  /* Will this ever be true? -N */
-  if (retrieved < 0)
-    USB_ERROR_STR(retrieved, "usb_bulk_read: error reading from endpoint %02x", ep);
+  if (result == kIOReturnSuccess)
+    /* wait for write to complete */
+    CFRunLoopRun();
+
+  CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
+
+  if (result != kIOReturnSuccess || retrieved < 0)
+    USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_read(ReadPipe): error reading from bulk endpoint %02x: %s",
+		  ep, darwin_error_str(result));
 
   return retrieved;
 }
@@ -796,18 +822,20 @@ int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
 		    int value, int index, char *bytes, int size, int timeout)
 {
   struct darwin_dev_handle *device = dev->impl_info;
+
+  io_return_t result;
+
 #if !defined (LIBUSB_NO_TIMEOUT_DEVICE)
   IOUSBDevRequestTO urequest;
 #else
   IOUSBDevRequest urequest;
 #endif
-  io_return_t result;
 
   if (usb_debug >= 3)
     fprintf(stderr, "usb_control_msg: %d %d %d %d %p %d %d\n",
             requesttype, request, value, index, bytes, size, timeout);
 
-  bzero(&urequest, sizeof(IOUSBDevRequestTO));
+  bzero(&urequest, sizeof(urequest));
 
   urequest.bmRequestType = requesttype;
   urequest.bRequest = request;
@@ -835,15 +863,13 @@ int usb_os_find_busses(struct usb_bus **busses)
 
   io_iterator_t deviceIterator;
   io_service_t usbDevice;
+  io_return_t result;
+  io_cf_plugin_ref_t *plugInInterface = NULL;
 
   usb_device_t **device;
 
-  IOCFPlugInInterface **plugInInterface = NULL;
-
-  io_return_t result;
-  SInt32 score;
-  UInt16 address;
-  UInt32 location;
+  s_int32_t score;
+  u_int32_t location;
 
   char buf[20];
   int i = 1;
@@ -871,7 +897,8 @@ int usb_os_find_busses(struct usb_bus **busses)
   matchingDict = NULL;
 
   if (result != kIOReturnSuccess)
-    USB_ERROR_STR(-darwin_to_errno (result), "usb_os_find_busses(IOServiceGetMatchingServices): %s", darwin_error_str(result));
+    USB_ERROR_STR(-darwin_to_errno (result), "usb_os_find_busses(IOServiceGetMatchingServices): %s",
+		  darwin_error_str(result));
 
 
   while (IOIteratorIsValid(deviceIterator) && (usbDevice = IOIteratorNext(deviceIterator))) {
@@ -935,23 +962,19 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
 
   io_iterator_t deviceIterator;
   io_service_t usbDevice;
+  io_return_t result;
+  io_cf_plugin_ref_t *plugInInterface = NULL;
 
   usb_device_t **device;
 
-  IOCFPlugInInterface **plugInInterface = NULL;
-
-  io_return_t result;
-  SInt32 score;
-  UInt16 address;
-  UInt32 location;
-  UInt32 bus_loc = bus->location;
+  s_int32_t score;
+  u_int16_t address;
+  u_int32_t location;
+  u_int32_t bus_loc = bus->location;
 
   /* for use in retrieving device description */
   IOUSBDevRequest req;
 
-  char buf[20];
-  int busnum = 0;
-
   /* a master port should have been created by usb_os_init */
   if (!masterPort)
     USB_ERROR(-ENOENT);
@@ -1049,10 +1072,16 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
   return 0;
 }
 
+int usb_os_determine_children(struct usb_bus *bus)
+{
+  /* Nothing yet */
+  return 0;
+}
+
 void usb_os_init(void)
 {
   if (masterPort == NULL) {
-    IOMasterPort(MACH_PORT_NULL, &masterPort);
+    IOMasterPort(masterPort, &masterPort);
     
     gNotifyPort = IONotificationPortCreate(masterPort);
   }
@@ -1060,16 +1089,16 @@ void usb_os_init(void)
 
 void usb_os_cleanup (void)
 {
-  if (masterPort != NULL) {
-    mach_port_deallocate(mach_task_self(), masterPort);
-    masterPort = NULL;
-  }
+  if (masterPort != NULL)
+    darwin_cleanup ();
 }
 
 int usb_resetep(usb_dev_handle *dev, unsigned int ep)
 {
   struct darwin_dev_handle *device;
+
   io_return_t result = -1;
+
   int pipeRef;
 
   if (!dev)
@@ -1096,7 +1125,9 @@ int usb_resetep(usb_dev_handle *dev, unsigned int ep)
 int usb_clear_halt(usb_dev_handle *dev, unsigned int ep)
 {
   struct darwin_dev_handle *device;
+
   io_return_t result = -1;
+
   int pipeRef;
 
   if (!dev)
@@ -1123,7 +1154,8 @@ int usb_clear_halt(usb_dev_handle *dev, unsigned int ep)
 int usb_reset(usb_dev_handle *dev)
 {
   struct darwin_dev_handle *device;
-  int result;
+  
+  io_return_t result;
 
   if (!dev)
     USB_ERROR(-ENXIO);
