@@ -20,21 +20,29 @@
 
 static char usb_path[PATH_MAX + 1] = "";
 
-int usb_os_open(usb_dev_handle *dev)
+static int device_open(struct usb_device *dev)
 {
   char filename[PATH_MAX + 1];
+  int fd;
 
   snprintf(filename, sizeof(filename) - 1, "%s/%s/%s",
-    usb_path, dev->bus->dirname, dev->device->filename);
+    usb_path, dev->bus->dirname, dev->filename);
 
-  dev->fd = open(filename, O_RDWR);
-  if (dev->fd < 0) {
-    dev->fd = open(filename, O_RDONLY);
-    if (dev->fd < 0)
+  fd = open(filename, O_RDWR);
+  if (fd < 0) {
+    fd = open(filename, O_RDONLY);
+    if (fd < 0)
       USB_ERROR_STR(-errno, "failed to open %s: %s",
 	filename, strerror(errno));
   }
 
+  return fd;
+}
+
+int usb_os_open(usb_dev_handle *dev)
+{
+  dev->fd = device_open(dev->device);
+
   return 0;
 }
 
@@ -118,66 +126,13 @@ int usb_set_altinterface(usb_dev_handle *dev, int alternate)
   return 0;
 }
 
-/* Linux usbdevfs has a limit of one page size per read/write. 4096 is */
-/* the most portable maximum we can do for now */
-#define MAX_READ_WRITE	4096
-
-int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int length,
-	int timeout)
-{
-  struct usb_bulktransfer bulk;
-  int ret, sent = 0;
-
-  /* Ensure the endpoint address is correct */
-  ep &= ~USB_ENDPOINT_IN;
-
-  do {
-    bulk.ep = ep;
-    bulk.len = length - sent;
-    if (bulk.len > MAX_READ_WRITE)
-      bulk.len = MAX_READ_WRITE;
-    bulk.timeout = timeout;
-    bulk.data = (unsigned char *)bytes + sent;
-
-    ret = ioctl(dev->fd, IOCTL_USB_BULK, &bulk);
-    if (ret < 0)
-      USB_ERROR_STR(-errno, "error writing to bulk endpoint %d: %s",
-	ep, strerror(errno));
-
-    sent += ret;
-  } while (ret > 0 && sent < length);
-
-  return sent;
-}
-
-int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
-	int timeout)
-{
-  struct usb_bulktransfer bulk;
-  int ret, retrieved = 0, requested;
-
-  /* Ensure the endpoint address is correct */
-  ep |= USB_ENDPOINT_IN;
-
-  do {
-    bulk.ep = ep;
-    requested = size - retrieved;
-    if (requested > MAX_READ_WRITE)
-      requested = MAX_READ_WRITE;
-    bulk.len = requested;
-    bulk.timeout = timeout;
-    bulk.data = (unsigned char *)bytes + retrieved;
-
-    ret = ioctl(dev->fd, IOCTL_USB_BULK, &bulk);
-    if (ret < 0)
-      USB_ERROR_STR(-errno, "error reading from bulk endpoint 0x%x: %s",
-	ep, strerror(errno));
-
-    retrieved += ret;
-  } while (ret > 0 && retrieved < size && ret == requested);
-
-  return retrieved;
-}
+/*
+ * Linux usbfs has a limit of one page size for synchronous bulk read/write.
+ * 4096 is the most portable maximum we can do for now.
+ * Linux usbfs has a limit of 16KB for the URB interface. We use this now
+ * to get better performance for USB 2.0 devices.
+ */
+#define MAX_READ_WRITE	(16 * 1024)
 
 int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
 	int value, int index, char *bytes, int size, int timeout)
@@ -202,64 +157,72 @@ int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
 }
 
 /* Reading and writing are the same except for the endpoint */
-static int usb_interrupt_transfer(usb_dev_handle *dev, int ep, char *bytes,
-	int size, int timeout)
+static int usb_urb_transfer(usb_dev_handle *dev, int ep, int urbtype,
+	char *bytes, int size, int timeout)
 {
-  struct usb_urb int_urb;
-  void *context;
-  int ret;
+  struct usb_urb urb;
   unsigned int bytesdone = 0, requested;
   struct timeval tv, tv_ref, tv_now;
-  int waiting;
+  void *context;
+  int ret, waiting;
 
   /*
-   * FIXME: Packetize large buffers here. 2.4 HCDs (atleast, haven't checked
-   * 2.5 HCDs yet) don't handle multi-packet Interrupt transfers. So we need
-   * to lookup the endpoint packet size and packetize appropriately here.
+   * FIXME: The use of the URB interface is incorrect here if there are
+   * multiple callers at the same time. We assume we're the only caller
+   * and if we get completions from another caller, this code will fail
+   * in interesting ways.
    */
 
-  /* get actual time, and add the timeout value. The result is the absolute
-   * time where we have to quit waiting for an interrupt message.
+  /*
+   * Get actual time, and add the timeout value. The result is the absolute
+   * time where we have to quit waiting for an message.
    */
   gettimeofday(&tv_ref, NULL);
   tv_ref.tv_sec = tv_ref.tv_sec + timeout / 1000;
   tv_ref.tv_usec = tv_ref.tv_usec + (timeout % 1000) * 1000;
 
-  if (tv_ref.tv_usec > 1e6) {
-    tv_ref.tv_usec -= 1e6;
+  if (tv_ref.tv_usec > 1000000) {
+    tv_ref.tv_usec -= 1000000;
     tv_ref.tv_sec++;
   }
 
   do {
+    fd_set writefds;
+
     requested = size - bytesdone;
     if (requested > MAX_READ_WRITE)
       requested = MAX_READ_WRITE;
 
-    int_urb.type = USB_URB_TYPE_INTERRUPT;
-    int_urb.endpoint = ep;
-    int_urb.flags = 0;
-    int_urb.buffer = bytes + bytesdone;
-    int_urb.buffer_length = requested;
-    int_urb.usercontext = (void *)ep;
-    int_urb.signr = 0;
-    int_urb.actual_length = 0;
+    urb.type = urbtype;
+    urb.endpoint = ep;
+    urb.flags = 0;
+    urb.buffer = bytes + bytesdone;
+    urb.buffer_length = requested;
+    urb.usercontext = (void *)ep;
+    urb.signr = 0;
+    urb.actual_length = 0;
+    urb.number_of_packets = 0;	/* don't do isochronous yet */
 
-    ret = ioctl(dev->fd, IOCTL_USB_SUBMITURB, &int_urb);
+    ret = ioctl(dev->fd, IOCTL_USB_SUBMITURB, &urb);
     if (ret < 0) {
       USB_ERROR_STR(-errno, "error submitting URB: %s", strerror(errno));
       return ret;
     }
 
+    FD_ZERO(&writefds);
+    FD_SET(dev->fd, &writefds);
+
     waiting = 1;
     while (((ret = ioctl(dev->fd, IOCTL_USB_REAPURBNDELAY, &context)) == -1) && waiting) {
       tv.tv_sec = 0;
       tv.tv_usec = 1000; // 1 msec
-      select(0, NULL, NULL, NULL, &tv); //sub second wait
+      select(dev->fd + 1, NULL, &writefds, NULL, &tv); //sub second wait
 
       /* compare with actual time, as the select timeout is not that precise */
       gettimeofday(&tv_now, NULL);
 
-      if ((tv_now.tv_sec >= tv_ref.tv_sec) && (tv_now.tv_usec >= tv_ref.tv_usec))
+      if ((tv_now.tv_sec > tv_ref.tv_sec) ||
+          ((tv_now.tv_sec == tv_ref.tv_sec) && (tv_now.tv_usec >= tv_ref.tv_usec)))
         waiting = 0;
     }
 
@@ -269,10 +232,10 @@ static int usb_interrupt_transfer(usb_dev_handle *dev, int ep, char *bytes,
      * error now
      */
     if (ret < 0 && errno != EAGAIN)
-      USB_ERROR_STR(-errno, "error reaping interrupt URB: %s", strerror(errno));
+      USB_ERROR_STR(-errno, "error reaping URB: %s", strerror(errno));
 
-    bytesdone += int_urb.actual_length;
-  } while (ret > 0 && bytesdone < size && ret == requested);
+    bytesdone += urb.actual_length;
+  } while (ret == 0 && bytesdone < size && urb.actual_length == requested);
 
   /* If the URB didn't complete in success or error, then let's unlink it */
   if (ret < 0) {
@@ -281,11 +244,18 @@ static int usb_interrupt_transfer(usb_dev_handle *dev, int ep, char *bytes,
     if (!waiting)
       rc = -ETIMEDOUT;
     else
-      rc = int_urb.status;
+      rc = urb.status;
 
-    ret = ioctl(dev->fd, IOCTL_USB_DISCARDURB, &int_urb);
+    ret = ioctl(dev->fd, IOCTL_USB_DISCARDURB, &urb);
     if (ret < 0 && errno != EINVAL && usb_debug >= 1)
-      fprintf(stderr, "error discarding interrupt URB: %s", strerror(errno));
+      fprintf(stderr, "error discarding URB: %s", strerror(errno));
+
+    /*
+     * When the URB is unlinked, it gets moved to the completed list and
+     * then we need to reap it or else the next time we call this function,
+     * we'll get the previous completion and exit early
+     */
+    ioctl(dev->fd, IOCTL_USB_REAPURB, &context);
 
     return rc;
   }
@@ -293,11 +263,33 @@ static int usb_interrupt_transfer(usb_dev_handle *dev, int ep, char *bytes,
   return bytesdone;
 }
 
+int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
+	int timeout)
+{
+  /* Ensure the endpoint address is correct */
+  return usb_urb_transfer(dev, ep, USB_URB_TYPE_BULK, bytes, size,
+		timeout);
+}
+
+int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
+	int timeout)
+{
+  /* Ensure the endpoint address is correct */
+  ep |= USB_ENDPOINT_IN;
+  return usb_urb_transfer(dev, ep, USB_URB_TYPE_BULK, bytes, size,
+		timeout);
+}
+
+/*
+ * FIXME: Packetize large buffers here. 2.4 HCDs (atleast, haven't checked
+ * 2.5 HCDs yet) don't handle multi-packet Interrupt transfers. So we need
+ * to lookup the endpoint packet size and packetize appropriately here.
+ */
 int usb_interrupt_write(usb_dev_handle *dev, int ep, char *bytes, int size,
 	int timeout)
 {
   /* Ensure the endpoint address is correct */
-  return usb_interrupt_transfer(dev, ep & ~USB_ENDPOINT_IN, bytes, size,
+  return usb_urb_transfer(dev, ep, USB_URB_TYPE_INTERRUPT, bytes, size,
 		timeout);
 }
 
@@ -305,7 +297,8 @@ int usb_interrupt_read(usb_dev_handle *dev, int ep, char *bytes, int size,
 	int timeout)
 {
   /* Ensure the endpoint address is correct */
-  return usb_interrupt_transfer(dev, ep | USB_ENDPOINT_IN, bytes, size,
+  ep |= USB_ENDPOINT_IN;
+  return usb_urb_transfer(dev, ep, USB_URB_TYPE_INTERRUPT, bytes, size,
 		timeout);
 }
 
@@ -373,6 +366,7 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
   while ((entry = readdir(dir)) != NULL) {
     char filename[PATH_MAX + 1];
     struct usb_device *dev;
+    struct usb_connectinfo connectinfo;
     int i, fd, ret;
 
     /* Skip anything starting with a . */
@@ -391,16 +385,27 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
     dev->filename[sizeof(dev->filename) - 1] = 0;
 
     snprintf(filename, sizeof(filename) - 1, "%s/%s", dirpath, entry->d_name);
-    fd = open(filename, O_RDONLY);
+    fd = open(filename, O_RDWR);
     if (fd < 0) {
-      if (usb_debug >= 2)
-        fprintf(stderr, "usb_os_find_devices: Couldn't open %s\n",
-		filename);
+      fd = open(filename, O_RDONLY);
+      if (fd < 0) {
+        if (usb_debug >= 2)
+          fprintf(stderr, "usb_os_find_devices: Couldn't open %s\n",
+                  filename);
 
-      free(dev);
-      continue;
+        free(dev);
+        continue;
+      }
     }
 
+    /* Get the device number */
+    ret = ioctl(fd, IOCTL_USB_CONNECTINFO, &connectinfo);
+    if (ret < 0) {
+      if (usb_debug)
+        fprintf(stderr, "usb_os_find_devices: couldn't get connect info\n");
+    } else
+      dev->devnum = connectinfo.devnum;
+
     ret = read(fd, (void *)&dev->descriptor, sizeof(dev->descriptor));
     if (ret < 0) {
       if (usb_debug)
@@ -498,6 +503,85 @@ err:
   return 0;
 }
 
+int usb_os_determine_children(struct usb_bus *bus)
+{
+  struct usb_device *dev, *devices[256];
+  struct usb_ioctl command;
+  int ret, i, i1;
+
+  /* Create a list of devices first */
+  memset(devices, 0, sizeof(devices));
+  for (dev = bus->devices; dev; dev = dev->next)
+    if (dev->devnum)
+      devices[dev->devnum] = dev;
+
+  /* Now fetch the children for each device */
+  for (dev = bus->devices; dev; dev = dev->next) {
+    struct usb_hub_portinfo portinfo;
+    int fd;
+
+    fd = device_open(dev);
+    if (fd < 0)
+      continue;
+
+    /* Query the hub driver for the children of this device */
+    if (dev->config && dev->config->interface && dev->config->interface->altsetting)
+      command.ifno = dev->config->interface->altsetting->bInterfaceNumber;
+    else
+      command.ifno = 0;
+    command.ioctl_code = IOCTL_USB_HUB_PORTINFO;
+    command.data = &portinfo;
+    ret = ioctl(fd, IOCTL_USB_IOCTL, &command);
+    if (ret < 0) {
+      /* errno == ENOSYS means the device probably wasn't a hub */
+      if (errno != ENOSYS && usb_debug > 1)
+        fprintf(stderr, "error obtaining child information: %s\n",
+		strerror(errno));
+
+      close(fd);
+      continue;
+    }
+
+    dev->num_children = 0;
+    for (i = 0; i < portinfo.numports; i++)
+      if (portinfo.port[i])
+        dev->num_children++;
+
+    dev->children = malloc(sizeof(struct usb_device *) * dev->num_children);
+    if (!dev->children) {
+      if (usb_debug > 1)
+        fprintf(stderr, "error allocating %d bytes memory for dev->children\n",
+                sizeof(struct usb_device *) * dev->num_children);
+
+      dev->num_children = 0;
+      close(fd);
+      continue;
+    }
+
+    for (i = 0, i1 = 0; i < portinfo.numports; i++) {
+      if (!portinfo.port[i])
+        continue;
+
+      dev->children[i1++] = devices[portinfo.port[i]];
+
+      devices[portinfo.port[i]] = NULL;
+    }
+
+    close(fd);
+  }
+
+  /*
+   * There should be one device left in the devices list and that should be
+   * the root device
+   */
+  for (i = 0; i < sizeof(devices) / sizeof(devices[0]); i++) {
+    if (devices[i])
+      bus->root_dev = devices[i];
+  }
+
+  return 0;
+}
+
 static int check_usb_vfs(const unsigned char *dirname)
 {
   DIR *dir;
